==> tslm-1.1.0.2-Server-img.tar.gz: tslm/migration/import_to_tslm_stage2.pl <==
#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.1 - 12/04/30 19:31:07
#
# @brief import_to_tslm_stage2.pl - Import data using ermmtool stage2.
#
#-----------------------------------------------------------------------

use strict;
use warnings;

#output datafles
my $libfile = "data/lib.dat";
my $appfile = "data/app.dat";
my $dbcartinlibfile="data/cartinlib.db2dat";
my $dbvolcartinlibfile="data/volcartinlib.db2dat";  
my $dbdrivefile="data/drive.db2dat"; 

my $tracefile="data/import_to_tslm2.log";

my @export_files=($libfile, $dbcartinlibfile, $dbvolcartinlibfile, $dbdrivefile, $appfile);

my $buf;
my $cmd; 
my $rc;

my $sec;
my $min;
my $hour;
my $mday;
my $mon;
my $year;
my $wday;
my $yday;
my $isdst;

sub show_copyright()
{
    print "IBM Tape System Library Manager\n";
    print "Licensed Materials - Property of IBM\n";
    print "(C) Copyright IBM Corp. 2004, 2012 All Rights Reserved\n";
    print "\n";
    sleep(1);
}

sub check_user()
{
    my $MYLOGIN = $ENV{'LOGNAME'}; 
    chomp($MYLOGIN); 
    #print $MYLOGIN . "\n";
    my $WHOAMI = `which whoami 2> /dev/null`;
    chomp($WHOAMI);
    #print $WHOAMI . "\n";
    if (-e $WHOAMI) {
        $MYLOGIN = `$WHOAMI 2> /dev/null`;
        chomp($MYLOGIN);
        #print $MYLOGIN . "xxxx\n";
    }
    if ($MYLOGIN ne "ermm") {
        print "This script must be run with user \"ermm\".\n";
        exit -1;
    }
}

sub create_data_directory
{
    mkdir ("data/");
    open(HANDLE,">>",$tracefile) or die "Can't open tracefile file $tracefile, RC=$! \n";
}

sub print_and_log()
{
    print "$_[0]\n"; print HANDLE "$_[0]\n";
}

sub quit_process()
{
    &print_and_log("");
    &print_and_log("Import to TSLM stage 2 did not complete successfully: RC=$_[0]");
    &print_and_log("");
    &print_and_log("-----");
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    &print_and_log("----- End of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
    &print_and_log("-----");
    &print_and_log("");

    close(HANDLE);

    exit $_[0];
}

sub check_export_files()
{
    &print_and_log("----- Checking exported files from eRMM exist in data directory...");
    foreach my $value (@export_files)
    {
        if (-f $value)
        {
            &print_and_log("$value found.");
        }
        else
        {
           &print_and_log("$value does not exist.");
           &print_and_log("Please make sure that all the exported files from eRMM are in data/ directory.");
           &quit_process(1);
        }
    }
}

sub check_ermmtool_connectivity {
    &print_and_log("----- Checking ermmtool is able to connect to Media Manager correctly...");
    $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin lssys >> $tracefile 2>&1";

    for (1..3)
    {
       $rc = system("$cmd");

       if ($rc eq 0) {
           &print_and_log("Done!");
           &print_and_log("");
           return;
       }
       else {
           &print_and_log("ermmtool lssys did not return expected outputs.");
           sleep(2);
       }
    }

    &print_and_log("");
    &print_and_log("Please make sure that TSLM CLI package is installed and Media Manager is running on this system.");

    &quit_process(2);
}

sub allocatecartinlib() {
    open IN, $dbcartinlibfile  or die "Can't open source file $dbcartinlibfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" ;

        # move carts into cartgroups
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin chcart $v1 -group $v2";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }

    open IN, $dbvolcartinlibfile  or die "Can't open source file $dbvolcartinlibfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" . "$v4 \n" . "$v5\n"  ;

        # allocate vols
        if ($v2 =~ /.*ERMMSystem.*/) {
            $cmd = "ermmtool -c $v2 -i ERMMAdmin alloc $v3 -pcl $v1";
        }
        else {
            $cmd = "ermmtool -c $v2 -i MigrationAI alloc $v3 -pcl $v1";
        }
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

sub setupdrive() {
    open IN, $dbdrivefile  or die "Can't open source file $dbdrivefile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" . "$v4 \n" . "$v5 \n" . "$v6 \n" ;

        # change drive attr to orig vals
 
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin chdrive $v1 -prio $v4 -group $v6 -sn $v5 ";
        if ($v3 =~ /.*0.*/) {
           $cmd = $cmd . "-disable ";
        }
        else {
           $cmd = $cmd . "-enable ";
        }
    
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

sub printandlogrc {
    my ($prc, $pcmd) = @_;
    my $success = "SUCCESS->";
    my $error =   "ERROR--->";
    my $ret =     "RC------>";
    my $buf;
    if ( ($prc eq "") || ($prc =~ /.*success.*/) ) {
        $buf = "$success ermmtool cmd:$pcmd --- successfully done.\n";
        &print_and_log($buf);
    }
    else {
        $buf = "$error ermmtool cmd:$pcmd --- execution error.";
        &print_and_log($buf);
        $buf = "$ret $prc ";
        &print_and_log($buf);
    }
}

sub auditlib() {
    open IN, $libfile  or die "Can't open source file $libfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        # skip header lines
        if ( ($buf =~ /.*Name.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";

        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n"  . "$v3 \n" . "$v4 \n"  . "$v5 \n"  . "$v6 \n";
 
        # add libs
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin audit $v1";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

sub postmig() {
    # remove temp MigrationAIs
    open IN, $appfile  or die "Can't open source file $appfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*Name.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4) = split(/ +/);
        # print "$v1 \n" . "$v2 \n"  . "$v3 \n" . "$v4 \n";
        
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin rmai MigrationAI -app $v1";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

#main
&show_copyright();
&check_user();
&create_data_directory();

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
&print_and_log("-----");
&print_and_log("----- Start of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
&print_and_log("-----");
&print_and_log("");

# export file check
&check_export_files();
# conectivity check
&check_ermmtool_connectivity();

# start import
&print_and_log("");
&print_and_log("----- Start importing data to TSLM...");
&print_and_log("");

#&auditlib();
&allocatecartinlib();
&setupdrive();
&postmig();

&print_and_log("");
&print_and_log("Import to TSLM completed successfully.");
&print_and_log("");
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
&print_and_log("-----");
&print_and_log("----- End of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
&print_and_log("-----");
&print_and_log("");

close(HANDLE);

exit(0);

==> tslm-1.2.0.1-Server-img.tar.gz: tslm/migration/import_to_tslm_stage2.pl <==
#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.4 - 12/12/25 22:11:32
#
# @brief import_to_tslm_stage2.pl - Import data using ermmtool stage2.
#
#-----------------------------------------------------------------------

use strict;
use warnings;

#output datafles
my $libfile = "data/lib.dat";
my $appfile = "data/app.dat";
my $dbcartinlibfile="data/cartinlib.db2dat";
my $dbvolcartinlibfile="data/volcartinlib.db2dat";  
my $dbdrivefile="data/drive.db2dat"; 
my $dgcfile="data/dgc.dat";
my $afgfile = "data/afg.dat";

my $tracefile="data/import_to_tslm2.log";

my @export_files=($libfile, $dbcartinlibfile, $dbvolcartinlibfile, $dbdrivefile, $appfile, $dgcfile, $afgfile);

my $buf;
my $cmd; 
my $rc;

my $sec;
my $min;
my $hour;
my $mday;
my $mon;
my $year;
my $wday;
my $yday;
my $isdst;

sub show_copyright()
{
    print "IBM Tape System Library Manager\n";
    print "Licensed Materials - Property of IBM\n";
    print "(C) Copyright IBM Corp. 2004, 2012 All Rights Reserved\n";
    print "\n";
    sleep(1);
}

sub check_user()
{
    my $MYLOGIN = $ENV{'LOGNAME'}; 
    chomp($MYLOGIN); 
    #print $MYLOGIN . "\n";
    my $WHOAMI = `which whoami 2> /dev/null`;
    chomp($WHOAMI);
    #print $WHOAMI . "\n";
    if (-e $WHOAMI) {
        $MYLOGIN = `$WHOAMI 2> /dev/null`;
        chomp($MYLOGIN);
        #print $MYLOGIN . "xxxx\n";
    }
    if ($MYLOGIN ne "ermm") {
        print "This script must be run with user \"ermm\".\n";
        exit -1;
    }
}

sub create_data_directory
{
    mkdir ("data/");
    open(HANDLE,">>",$tracefile) or die "Can't open tracefile file $tracefile, RC=$! \n";
}

sub print_and_log()
{
    print "$_[0]\n"; print HANDLE "$_[0]\n";
}

sub quit_process()
{
    &print_and_log("");
    &print_and_log("Import to TSLM stage 2 did not complete successfully: RC=$_[0]");
    &print_and_log("");
    &print_and_log("-----");
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    &print_and_log("----- End of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
    &print_and_log("-----");
    &print_and_log("");

    close(HANDLE);

    exit $_[0];
}

sub check_export_files()
{
    &print_and_log("----- Checking exported files from eRMM exist in data directory...");
    foreach my $value (@export_files)
    {
        if (-f $value)
        {
            &print_and_log("$value found.");
        }
        else
        {
           &print_and_log("$value does not exist.");
           &print_and_log("Please make sure that all the exported files from eRMM are in data/ directory.");
           &quit_process(1);
        }
    }
}

sub check_ermmtool_connectivity {
    &print_and_log("----- Checking ermmtool is able to connect to Media Manager correctly...");
    $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin lssys >> $tracefile 2>&1";

    for (1..3)
    {
       $rc = system("$cmd");

       if ($rc eq 0) {
           &print_and_log("Done!");
           &print_and_log("");
           return;
       }
       else {
           &print_and_log("ermmtool lssys did not return expected outputs.");
           sleep(2);
       }
    }

    &print_and_log("");
    &print_and_log("Please make sure that TSLM CLI package is installed and Media Manager is running on this system.");

    &quit_process(2);
}

sub allocatecartinlib() {
    my $count = 0;
    open IN, $dbcartinlibfile  or die "Can't open source file $dbcartinlibfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" ;

        # move carts into cartgroups
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin chcart $v1 -group $v2";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
        $count++;
    }

    # when no data found
    if ($count == 0) {
        print HANDLE "No data found in $dbcartinlibfile\n";
    }

    $count = 0;
    open IN, $dbvolcartinlibfile  or die "Can't open source file $dbvolcartinlibfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" . "$v4 \n" . "$v5\n"  ;

        # allocate vols
        if ($v2 =~ /.*ERMMSystem.*/) {
            $cmd = "ermmtool -c $v2 -i ERMMAdmin alloc $v3 -pcl $v1";
        }
        else {
            $cmd = "ermmtool -c $v2 -i MigrationAI alloc $v3 -pcl $v1";
        }
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
        $count++;
    }

    # when no data found
    if ($count == 0) {
        print HANDLE "No data found in $dbvolcartinlibfile\n";
    }
}

sub setupdrive() {
    my $count = 0;
    open IN, $dbdrivefile  or die "Can't open source file $dbdrivefile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*ID.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n" . "$v3 \n" . "$v4 \n" . "$v5 \n" . "$v6 \n" ;

        # change drive attr to orig vals
 
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin chdrive $v1 -prio $v4 -group $v6 -sn $v5 ";
        if ($v3 =~ /.*0.*/) {
           $cmd = $cmd . "-disable ";
        }
        else {
           $cmd = $cmd . "-enable ";
        }
    
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
        $count++;
    }

    # when no data found
    if ($count == 0) {
        print HANDLE "No data found in $dbdrivefile\n";
    }
}

sub setdg2c() {
    my $count = 0;
    open IN, $dgcfile or die "Can't open source file $dgcfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        my (@v1) = split(/text\[/,$buf);
        foreach my $rec (@v1) {
            if ($rec =~ /.*response.*/) {
                next;
            }
            chomp($rec);
      
            my @da = createdataarr($rec);
       
            # set DeallocGroup to Cartridge

            if ($da[1] ne "")
            {
                $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin chcart $da[0] -dealloc $da[1]";
                $rc = `$cmd`;
                printandlogrc($rc,$cmd);
                $count++;
            }
        }
    }

    # when no data found
    if ($count == 0) {
        print HANDLE "No data found in $dgcfile\n";
    }
}

sub setafg2c() {
    my $count = 0;
    open IN, $afgfile or die "Can't open source file $afgfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        my (@v1) = split(/text\[/,$buf);
        foreach my $rec (@v1) {
            if ($rec =~ /.*response.*/) {
                next;
            }
            chomp($rec);
      
            my @da = createdataarr($rec);
       
            # set AllocatedFromGroup to Cartridge

            if ($da[1] ne "")
            {
                $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin raw attribute task[\\\"migtaskid\\\"] set[Cartridge.\\\"AllocatedFromGroup\\\" \\\"$da[1]\\\"] match[streq\\(Cartridge.\\\"CartridgePCL\\\" \\\"$da[0]\\\"\\)] ";
                $rc = `$cmd`;
                printandlogrc($rc,$cmd);
                $count++;
            }
        }
    }

    # when no data found
    if ($count == 0) {
        print HANDLE "No data found in $afgfile\n";
    }
}

sub createdataarr() {
    my ($buf) = @_;
   
    my @str;
    @str = split(/ +/,$buf);
    my $i = 0;
    foreach my $x (@str) {
        $str[$i] =~ s/\"//;
        $str[$i] =~ s/\"//;
        $i++; 
    }
    $str[$i-1] =~ s/\]//;
    return @str
}

sub printandlogrc {
    my ($prc, $pcmd) = @_;
    my $success = "SUCCESS->";
    my $error =   "ERROR--->";
    my $ret =     "RC------>";
    my $buf;
    if ( ($prc eq "") || ($prc =~ /.*success.*/) ) {
        $buf = "$success ermmtool cmd:$pcmd --- successfully done.\n";
        &print_and_log($buf);
    }
    else {
        $buf = "$error ermmtool cmd:$pcmd --- execution error.";
        &print_and_log($buf);
        $buf = "$ret $prc ";
        &print_and_log($buf);
    }
}

sub auditlib() {
    open IN, $libfile  or die "Can't open source file $libfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        # skip header lines
        if ( ($buf =~ /.*Name.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";

        my ($v1, $v2, $v3, $v4, $v5, $v6) = split(/ +/);
        # print "$v1 \n" . "$v2 \n"  . "$v3 \n" . "$v4 \n"  . "$v5 \n"  . "$v6 \n";
 
        # add libs
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin audit $v1";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

sub postmig() {
    # remove temp MigrationAIs
    open IN, $appfile  or die "Can't open source file $appfile: $! \n"; 
    while (<IN>) {
        $buf = $_;
        if ( ($buf =~ /.*Name.*/) || ($buf =~ /.*=====.*/) ) {
            next;
        }
        # print "$buf";
        my ($v1, $v2, $v3, $v4) = split(/ +/);
        # print "$v1 \n" . "$v2 \n"  . "$v3 \n" . "$v4 \n";
        
        $cmd = "ermmtool -c ERMMSystem -i ERMMAdmin rmai MigrationAI -app $v1";
        $rc = `$cmd`;
        printandlogrc($rc,$cmd);
    }
}

#main
&show_copyright();
&check_user();
&create_data_directory();

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
&print_and_log("-----");
&print_and_log("----- Start of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
&print_and_log("-----");
&print_and_log("");

# export file check
&check_export_files();
# conectivity check
&check_ermmtool_connectivity();

# start import
&print_and_log("");
&print_and_log("----- Start importing data to TSLM...");
&print_and_log("");

#&auditlib();
&allocatecartinlib();
&setupdrive();
&setdg2c();
&setafg2c();
&postmig();

&print_and_log("");
&print_and_log("Import to TSLM completed successfully.");
&print_and_log("");
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
&print_and_log("-----");
&print_and_log("----- End of import to TSLM stage 2 on $year/$mon/$mday $hour:$min:$sec");
&print_and_log("-----");
&print_and_log("");

close(HANDLE);

exit(0);
